package service

import (
	"context"
	"doc-service/domain/entity/serversitemicros/docserverproject"
	service "doc-service/domain/repository"
	"doc-service/infra/db"
	"github.com/nacos-group/nacos-sdk-go/v2/common/logger"
	pb "gitlab.galaxy-immi.com/Backend-group/proto/pb/docservice"
	"time"
)

type DocServiceService struct {
	pb.UnimplementedDocServiceServiceServer
}

// 发起文件转换
func (doc *DocServiceService) DoConversion(c context.Context, req *pb.DoConversionRequest) (resp *pb.DoConversionResponse, err error) {

	resp = &pb.DoConversionResponse{}

	request := &service.DoConversionRequest{
		ProjectName:  req.ProjectName,
		SourceType:   req.SourceType,
		TargetType:   req.TargetType,
		SourceURI:    req.SourceUrl,
		TargetURI:    req.TargetUrl,
		IsPrivate:    req.IsPrivate,
		CallbackData: req.CallbackData,
		CallbackUrl:  req.CallbackUrl,
	}

	//创建项目
	createErr := service.NewConversion().CreateProject()
	if createErr != nil {
		resp.Msg = createErr.Error()
		resp.Code = 100010
		resp.Data = nil
		return resp, err
	}

	//入库
	isPri := 0
	if req.IsPrivate {
		isPri = 1
	}
	save, errsInfo := db.GetServerSiteMicrosDB().DocServerProject.Create().
		SetProjectName(req.ProjectName).
		SetCreatedAt(time.Now()).
		SetSourceURL(request.SourceURI).
		SetSourceType(req.SourceType).
		SetTargetType(req.TargetType).
		SetIsPrivate(int8(isPri)).
		SetTargetURL(request.TargetURI).
		Save(c)

	if errsInfo != nil {
		resp.Msg = errsInfo.Error()
		resp.Code = 100010
		resp.Data = nil
		return resp, errsInfo
	}

	//添加回调
	back, errBack := db.GetServerSiteMicrosDB().DocServerCallback.Create().
		SetCreatedAt(time.Now()).
		SetCallbackData(request.CallbackData).
		SetCallbackURL(request.CallbackUrl).
		SetStatus(0).
		SetServerProjectID(save.ID).
		Save(c)

	if errBack != nil {
		resp.Msg = errBack.Error()
		resp.Code = 100010
		resp.Data = nil
		return resp, errBack
	}

	resp.Msg = "success"
	resp.Code = 200
	resp.Data = &pb.DoConversionData{
		Id:         save.ID,
		TaskId:     "",
		CallbackId: back.ID,
	}
	return resp, nil
}

// 获取转换结果
func (doc *DocServiceService) GetProjectInfo(c context.Context, req *pb.GetProjectRequest) (resp *pb.GetProjectResponse, err error) {
	resp = &pb.GetProjectResponse{}
	ConversionInfo := &pb.DoConversionInfo{}

	request := &service.GetTaskRequest{
		ProjectId:         req.ProjectId,
		ProjectName:       req.ProjectName,
		TaskType:          req.TaskType,
		TaskId:            req.TaskId,
		RequestDefinition: req.RequestDefinition,
	}
	request.RequestDefinition = true
	if req.ProjectId > 0 {
		ConversionInfo, infoErr := db.GetServerSiteMicrosDB().DocServerProject.Get(c, req.ProjectId)
		if infoErr != nil {
			resp.Msg = err.Error()
			resp.Code = 100010
			resp.Data = nil
			return resp, err
		}
		request.ProjectName = ConversionInfo.ProjectName
		request.TaskId = ConversionInfo.TaskID
		request.TaskType = ConversionInfo.TargetType
	} else {
		projectData, infoErr := db.GetServerSiteMicrosDB().DocServerProject.Query().Where(
			docserverproject.ProjectNameEQ(request.ProjectName),
			docserverproject.TargetTypeEQ(request.TaskType),
			docserverproject.TaskIDEQ(request.TaskId),
		).First(c)
		if infoErr != nil {
			resp.Msg = err.Error()
			resp.Code = 100010
			resp.Data = nil
			return resp, err
		}
		ConversionInfo.Id = projectData.ID
		ConversionInfo.ProjectName = projectData.ProjectName
		ConversionInfo.SourceType = projectData.SourceType
		ConversionInfo.TargetType = projectData.TargetType
		ConversionInfo.SourceUrl = *projectData.SourceURL
		ConversionInfo.TargetUrl = *projectData.TargetURL
		ConversionInfo.TaskId = projectData.TaskID
	}

	result, errs := service.NewConversion().GetTaskProject(*request)
	logger.Info("返回数据", result)
	if errs != nil {
		resp.Msg = err.Error()
		resp.Code = 100010
		resp.Data = nil
		return resp, err
	}
	resp.Msg = "success"
	resp.Code = 200
	resp.Data = ConversionInfo
	return resp, nil
}
